@@ -1,5 +1,5 @@
-; ACL2 Version 2.9.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2005  University of Texas at Austin
+; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -26,7 +26,9 @@
 
 (in-package "ACL2")
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            SUPPORT FOR NON-STANDARD ANALYSIS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; Amazingly, some lisps do not have a definition for realp.  In those
 ; implementations (apparently including at least one early version of GCL), we
@@ -45,7 +47,9 @@
 (if (not (fboundp 'lisp::realp))
     (setf (symbol-function 'lisp::realp) (symbol-function 'acl2-realp)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            PROCLAIMING
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun macroexpand-till (form sym)
 
@@ -107,22 +111,30 @@
 (defun max-output-type-for-declare-form (type1 type2)
 
 ; We return a supertype of type1 and type2, preferably as small as possible,
-; else nil.  We assume that type1 is not null.
+; else nil.  We assume that each typei that is not null is (values ...) or is
+; some sort of integer type.
 
   (cond
    ((equal type1 type2)
     type1)
-   ((null type2)
+   ((or (null type1)
+        (null type2))
     nil)
    ((or (eq type1 'integer)
         (eq type2 'integer)
-        (and (eq (car type1) 'integer)
-             (null (cddr type1)))
-        (and (eq (car type2) 'integer)
-             (null (cddr type2)))
-        (member '* (cdr type1) :test 'eq)
-        (member '* (cdr type2) :test 'eq))
+        (and (consp type1)
+             (eq (car type1) 'integer)
+             (or (and (consp (cdr type1))
+                      (null (cddr type1)))
+                 (member '* (cdr type1) :test 'eq)))
+        (and (consp type2)
+             (eq (car type2) 'integer)
+             (or (and (consp (cdr type2))
+                      (null (cddr type2)))
+                 (member '* (cdr type2) :test 'eq))))
     'integer)
+   ((or (atom type1) (atom type2))
+    nil)
    (t (let ((sym1 (car type1))
             (sym2 (car type2)))
         (cond
@@ -136,10 +148,16 @@
              (list 'integer
                    (min  (cadr type1)  (cadr type2))
                    (max (caddr type1) (caddr type2))))
+            (values
+             (cons 'values (max-output-type-for-declare-form-lst (cdr type1)
+                                                                 (cdr type2))))
             (otherwise
              (error
               "Unexpected type for max-output-type-for-declare-form: ~s"
               type1))))
+         ((or (eq sym1 'values)
+              (eq sym2 'values))
+          nil)
          (t (let* ((pair1 (convert-type-to-integer-pair type1))
                    (pair2 (convert-type-to-integer-pair type2))
                    (lower1 (car pair1))
@@ -156,6 +174,23 @@
                       (min lower1 lower2)
                       (max upper1 upper2)))))))))))
 
+#+(and (or akcl openmcl) acl2-mv-as-values)
+(defun max-output-type-for-declare-form-lst (type-list1 type-list2)
+  (cond ((or (atom type-list1) (atom type-list2))
+         (cond
+          ((and (atom type-list1) (atom type-list2))
+           nil)
+          (t
+           (error "Implementation error:~%~
+                   max-output-type-for-declare-form-lst called on lists of~%~
+                   different length:~%~
+                   ~s~%  ~s~%~
+                   Please contact the ACL2 implementors."))))
+        (t (cons (max-output-type-for-declare-form
+                  (car type-list1) (car type-list2))
+                 (max-output-type-for-declare-form-lst
+                  (cdr type-list1) (cdr type-list2))))))
+
 #+(or akcl openmcl)
 (defun output-type-for-declare-form-rec (form)
 
@@ -169,46 +204,80 @@
     nil)
    ((eq (car form) 'the)
     (let ((typ (cadr form)))
-      (and (or (eq typ 'integer)
-               (and (consp typ)
-                    (member (car typ)
-                            '(integer signed-byte unsigned-byte)
-                            :test 'equal)
-                    typ)))))
+      (cond ((eq typ 'integer)
+             typ)
+            ((and (consp typ)
+                  (member (car typ)
+                          '(integer signed-byte unsigned-byte values)))
+             typ))))
    ((eq (car form) 'if)
     (let ((type1 (output-type-for-declare-form-rec (caddr form))))
-      (and type1
+      (and type1 ; optimization
            (max-output-type-for-declare-form
             type1
             (output-type-for-declare-form-rec (cadddr form))))))
+   #+acl2-mv-as-values
+   ((eq (car form) 'values)
+    (cons 'values (output-type-for-declare-form-rec-list (cdr form))))
    (t (multiple-value-bind
        (new-form flg)
        (macroexpand-1 form)
        (cond ((null flg) nil)
              (t (output-type-for-declare-form-rec new-form)))))))
        
+#+(and (or akcl openmcl) acl2-mv-as-values)
+(defun output-type-for-declare-form-rec-list (forms)
+  (cond ((atom forms)
+         nil)
+        (t (cons (or (output-type-for-declare-form-rec (car forms))
+                     t)
+                 (output-type-for-declare-form-rec-list (cdr forms))))))
+
 #+(or akcl openmcl)
 (defun output-types-for-declare-form (fn form)
+
+; We return a list of output types, one per value.  So if #-acl2-mv-as-values,
+; then we always return a list of length one.
+
   #-acl2-mv-as-values
   (declare (ignore fn))
   #-acl2-mv-as-values
   (list (or (output-type-for-declare-form-rec form)
             t))
   #+acl2-mv-as-values
-  (let ((stobjs-out
+  (let ((result (output-type-for-declare-form-rec form))
+        (stobjs-out
          (eval `(let ((wrld (w *the-live-state*)))
                   (and wrld ; cheap check to avoid any early proclaim-files
-                       (getprop ',fn 'stobjs-out nil 'current-acl2-world wrld))))))
-    (and stobjs-out
-         (cons (or (output-type-for-declare-form-rec form)
-                   t)
-
-; There is certainly potential for improvement here, where we look for types on
-; all values instead of just the first.  But for that we would need to allow
-; (the (values type1 type2 ...) form) in ACL2.
-
-               (make-list (length (cdr stobjs-out))
-                          :initial-element t)))))
+                       (getprop ',fn 'stobjs-out nil 'current-acl2-world
+                                wrld))))))
+    (when (and stobjs-out
+               result
+               (or (and (consp result)
+                        (eq (car result) 'values)
+                        (null (cdr stobjs-out)))
+                   (and (or (atom result)
+                            (not (eq (car result) 'values)))
+                        (cdr stobjs-out))))
+      (error "Implementation error in output-types-for-declare-form:~%~
+              stobjs-out and result don't mesh.~%~
+              Stobjs-out = ~s~%~
+              Result = ~s~%~
+              Please contact the ACL2 implementors."
+             stobjs-out result))
+    (cond
+     ((and (consp result)
+           (eq (car result) 'values))
+      (cdr result))
+     (result
+      (list result))
+     (stobjs-out
+      (make-list (length stobjs-out) :initial-element t))
+; We allow this function to be called even on functions for which there is null
+; stobjs-out (hence, functions defined only in raw Lisp), but if no output type
+; is computed then with stobjs-out equal to nil we cannot compute an output
+; type, so we return nil.
+     )))
 
 (defun make-defun-declare-form (fn form)
 
@@ -223,12 +292,12 @@
          (let ((formals (caddr form)))
            (cond
             ((null (intersection formals lambda-list-keywords))
-             `(proclaim (quote (ftype (function
-                                       ,(arg-declarations
-                                         formals
-                                         (collect-types (cdddr form)))
-                                       ,@output-types)
-                                      ,(cadr form)))))
+             `(declaim (ftype (function
+                               ,(arg-declarations
+                                 formals
+                                 (collect-types (cdddr form)))
+                               ,@output-types)
+                              ,(cadr form))))
             (t nil))))))
 
 #+(or akcl openmcl)
@@ -243,10 +312,10 @@
                             (t nil))))
     (cond
      (output-type
-      `(proclaim (quote (type ,output-type ,(cadr form)))))
+      `(declaim (type ,output-type ,(cadr form))))
      (t (let ((val (symbol-value (cadr form))))
           (if (integerp val)
-              `(proclaim (quote (type (integer ,val ,val) ,(cadr form))))
+              `(declaim (type (integer ,val ,val) ,(cadr form)))
             nil))))))
 
 #+(or akcl openmcl)
@@ -356,7 +425,9 @@ column.
               (t (proclaim-form form stream)))))
      nil)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;          ACL2's Implementation of the Backquote Readmacro
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defparameter *comma* (make-symbol "COMMA")
   "*comma* is used by the backquote reader.  When the reader
@@ -437,7 +508,9 @@ notation causes an error and (b) the use of ,. is not permitted."
                (backquote (car l))
                (backquote-lst (cdr l))))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            SUPPORT FOR ACL2 CHARACTER READER
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun rev1@ (x acc)
 
@@ -516,7 +589,9 @@ notation causes an error and (b) the use of ,. is not permitted."
   (let ((ch (read-char s)))
     (acl2-read-character-string s ch)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            DUAL PACKAGES
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; The following function used to be defined in axioms.lisp (with
 ; #-acl2-loop-only), but we need it here.
@@ -529,7 +604,7 @@ notation causes an error and (b) the use of ,. is not permitted."
         ((let ((p (symbol-package x)))
 
 ; The following code should be kept in sync with the gcl-only code in the
-; vacinity of the calls of rename-package in acl2.lisp, and with the definition
+; vicinity of the calls of rename-package in acl2.lisp, and with the definition
 ; of *main-lisp-package-name*.  The point is that the low-level package name
 ; for Lisp symbols is "LISP" in GCL but we want to treat it as "COMMON-LISP" in
 ; all Lisp implementations.  Thus, the extra test for the *main-lisp-package*
@@ -586,17 +661,47 @@ notation causes an error and (b) the use of ,. is not permitted."
 (defmacro gv (fn args val)
   (sublis `((funny-fn . ,fn)
             (funny-args . ,args))
-          `(if (or (f-get-global 'guard-checking-on *the-live-state*)
-                   (f-get-global 'safe-mode *the-live-state*))
-               (throw-raw-ev-fncall
-                (list 'ev-fncall-guard-er
-                      'funny-fn
-                      ,(cons 'list 'funny-args)
-                      (guard 'funny-fn nil (w *the-live-state*))
+          `(let ((gc-on
+                  (not (member (f-get-global 'guard-checking-on
+                                             *the-live-state*)
+                               '(nil :none)
+                               :test 'eq))))
+             (if (or gc-on
+                     (f-get-global 'safe-mode *the-live-state*))
+                 (throw-raw-ev-fncall
+                  (list 'ev-fncall-guard-er
+                        'funny-fn
+                        ,(cons 'list 'funny-args)
+                        (guard 'funny-fn nil (w *the-live-state*))
 
 ; We assume that the primitives don't take state.
 
-                      nil
-                      (not (f-get-global 'guard-checking-on
-                                         *the-live-state*))))
-             ,val)))
+                        nil
+                        (not gc-on)))
+               ,val))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;                            ENVIRONMENT SUPPORT
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+; The following is first used in acl2-init.lisp, so we define it here.
+
+(defun getenv$ (string)
+
+; The following either returns the value of the given environment variable or
+; returns nil (in lisps where we do not yet know how to get that value).
+
+  (or #+cmu
+      (and (boundp ext::*environment-list*)
+           (cdr (assoc-eq (intern string :keyword)
+                          ext::*environment-list*)))
+      #+(or gcl allegro lispworks openmcl sbcl clisp)
+      (let ((fn
+             #+gcl       'si::getenv
+             #+allegro   'sys::getenv
+             #+lispworks 'cl::getenv
+             #+openmcl   'ccl::getenv
+             #+sbcl      'sb-ext::posix-getenv
+             #+clisp     'ext:getenv))
+        (and (fboundp fn)
+             (funcall fn string)))))
