@@ -1,5 +1,5 @@
-; ACL2 Version 2.9 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2004  University of Texas at Austin
+; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic NOTE-2-0.
@@ -34,6 +34,10 @@
 
 ; This file cannot be compiled because it changes packages in the middle.
 
+; Allow taking advantage of threads in SBCL and OpenMCL.
+#+(or (and sbcl sb-thread) openmcl)
+(push :acl2-mv-as-values *features*)
+
 #+akcl
 (setq si:*notify-gbc* t)
 
@@ -53,7 +57,7 @@
 ; with the redefinition of si::break-level.
 
 #+akcl
-(proclaim '(special si::arglist))
+(declaim (special si::arglist))
 
 #+akcl
 (let ((v (symbol-function 'si::break-level)))
@@ -78,48 +82,62 @@
      (dolist
       (x arguments)
       (setq result (concatenate 'string result " " x)))
-     result)
-   "/bin/sh")
-  #+allegro (excl::shell 
-             (let ((result string))
-               (dolist
-                (x arguments)
-                (setq result (concatenate 'string result " " x)))
-               result))
+     result))
+  #+allegro
+  (let ((result string))
+    (dolist
+      (x arguments)
+      (setq result (concatenate 'string result " " x)))
+    #-unix
+    (excl::shell result)
+    #+unix
+
+; In Allegro CL in Unix, we can avoid spawning a new shell by calling run-shell-command
+; on a simple vector.  So we parse the resulting string "cmd arg1 ... argk" and
+; run with the simple vector #(cmd cmd arg1 ... argk).
+
+    (excl::run-shell-command
+     (let ((lst nil)
+           (len (length result))
+           (n 0))
+       (loop
+        (if (>= n len) (return)) ; else get next word
+        (let ((start n)
+              (ch (char result n)))
+          (cond
+           ((member ch '(#\Space #\Tab))
+            (setq n (1+ n)))
+           (t (loop
+               (if (or (>= n len)
+                       (member (setq ch (char result n))
+                               '(#\Space #\Tab)))
+                   (return)
+                 (setq n (1+ n))))
+              (setq lst (cons (subseq result start n)
+                              lst))))))
+       (setq result (nreverse lst))
+       (setq result (coerce (cons (car result) result) 'vector)))))
   #+cmu
   (common-lisp-user::run-program string arguments :output t)
+  #+sbcl
+  (sb-ext:run-program string arguments :output t :search t)
   #+clisp
   (ext:run-program string :arguments arguments)
   #+openmcl
-  (ccl::run-program string arguments)
-  #-(or akcl lucid lispworks allegro cmu clisp openmcl)
+  (ccl::run-program string arguments :output t)
+  #-(or akcl lucid lispworks allegro cmu sbcl clisp openmcl)
   (declare (ignore string arguments))
-  #-(or akcl lucid lispworks allegro cmu clisp openmcl)
+  #-(or akcl lucid lispworks allegro cmu sbcl clisp openmcl)
   (error "SYSTEM-CALL is not yet defined in this Lisp."))
 
 (defun copy-acl2 (dir)
   (system-call
    "cp"
-   #+cmu
    (append '("makefile"
              "acl2.lisp"
              "acl2-check.lisp"
              "acl2-fns.lisp"
-             "init.lsp"
-             "acl2-init.lisp")
-           (append (let ((result (list (format nil "~a" dir))))
-                     (dolist
-                      (x *copy-of-acl2-files*)
-                      (setq result
-                            (cons (format nil "~a.lisp" x)
-                                  result)))
-                     result)))
-   #-cmu
-   (append '("makefile"
-             "acl2.lisp"
-             "acl2-check.lisp"
-             "acl2-fns.lisp"
-             "init.lsp"
+             "init.lisp"
              "acl2-init.lisp")
            (append (let ((result (list (format nil "~a" dir))))
                      (dolist
@@ -315,19 +333,29 @@
                                target-directory
                                dir)))))))))
 
-    (format t "Finished creating a command file for copying distribution files."
-            source-directory target-directory)))
+    (format t "Finished creating a command file for copying distribution files.")))
 
 (defun make-tags ()
+  #-(or openmcl sbcl cmu)
+; We disallow openmcl and sbcl for the following check.  We have found that the
+; result of the system-call is a process, (typep <result> 'external-process) in
+; openmcl and (typep <result> 'sb-impl::process) in sbcl, which can probably be
+; made to yield the status.  But the status is 0 even for commands not found,
+; so why bother?  Since cmucl seems to fall victim in the same way as cmucl, we
+; treat these two the same here.
+  (when (not (eql (system-call "which" '("etags")) 0))
+    (format t "SKIPPING etags: No such program is in the path.")
+    (return-from make-tags 1))
   (system-call "etags"
-               #+(or cmu clisp openmcl)
+               #+(or cmu sbcl clisp openmcl)
                (append '("acl2.lisp"
                          "acl2-check.lisp"
                          "acl2-fns.lisp"
-                         "init.lsp"
+                         "init.lisp"
                          "acl2-init.lisp"
                          "akcl-acl2-trace.lisp"
-                         "allegro-acl2-trace.lisp")
+                         "allegro-acl2-trace.lisp"
+                         "openmcl-acl2-trace.lisp")
                        (let ((result nil))
                          (dolist
                           (x *copy-of-acl2-files*)
@@ -335,11 +363,11 @@
                                 (cons (format nil "~a.lisp" x)
                                       result)))
                          (reverse result)))
-               #-(or cmu clisp openmcl)
+               #-(or cmu sbcl clisp openmcl)
                (append '("acl2.lisp"
                          "acl2-check.lisp"
                          "acl2-fns.lisp"
-                         "init.lsp"
+                         "init.lisp"
                          "acl2-init.lisp"
                          "akcl-acl2-trace.lisp"
                          "allegro-acl2-trace.lisp")
@@ -358,7 +386,7 @@
   (concatenate
    'string
    "~% ~a built ~a.~
-   ~% Copyright (C) 2004  University of Texas at Austin~
+   ~% Copyright (C) 2006  University of Texas at Austin~
    ~% ACL2 comes with ABSOLUTELY NO WARRANTY.  This is free software and you~
    ~% are welcome to redistribute it under certain conditions.  For details,~
    ~% see the GNU General Public License.~%~
@@ -367,7 +395,9 @@
    "~% Type (LP) to enter the ACL2 command loop;~
     ~% then see the documentation topic note-2-~a for recent changes.~%"
    #-lispworks
-   "~% See the documentation topic note-2-~a for recent changes.~%"
+   "~% See the documentation topic ~a for recent changes.~
+    ~% Note: We have modified the prompt in some underlying Lisps to further~
+    ~% distinguish it from the ACL2 prompt.~%"
 
    "~% NOTE!!  Proof trees are disabled in ACL2.  To enable them in emacs,~
     ~% look under the ACL2 source directory in interface/emacs/README.doc; ~
@@ -378,7 +408,7 @@
 
 ; There is not a true notion of home directory for Windows systems, as far as
 ; we know.  We may provide one at a future point, but for now, we simply act as
-; though ~/acl2-init.lisp does not exist on such systems.
+; though ~/acl2-init.lsp does not exist on such systems.
 
 
   #+mswindows
@@ -395,24 +425,85 @@
 ; major.minor.extra.  The converse holds for versions of GCL past perhaps 2.0.
 
   (and (boundp 'si::*gcl-major-version*)
+       (integerp si::*gcl-major-version*)
        (if (= si::*gcl-major-version* major)
            (and (boundp 'si::*gcl-minor-version*)
+                (integerp si::*gcl-minor-version*)
                 (if (= si::*gcl-minor-version* minor)
                     (and (boundp 'si::*gcl-extra-version*)
+                         (integerp si::*gcl-extra-version*)
                          (> si::*gcl-extra-version* extra))
                   (> si::*gcl-minor-version* minor)))
          (> si::*gcl-major-version* major))))
 
+(defun chmod-executable (sysout-name)
+  (system-call "chmod" (list "+x" sysout-name)))
+
+(defmacro write-exec-file (stream string &rest args)
+  `(progn (format ,stream
+                  "#!/bin/sh~%~a~%"
+                  (let ((books-dir (getenv$ "ACL2_SYSTEM_BOOKS")))
+                    (if books-dir
+                        (format nil "export ACL2_SYSTEM_BOOKS=~a" books-dir)
+                      "")))
+          (format ,stream ,string ,@args)))
+
+#+akcl
+(defun save-acl2-in-akcl-aux (sysout-name gcl-exec-name
+                                          write-worklispext
+                                          set-optimize-maximum-pages)
+  (if (and write-worklispext (probe-file "worklispext"))
+      (delete-file "worklispext"))
+  (let* ((ext "gcl")
+         (ext+
+
+; We deal with the apparent fact that Windows implementations of GCL append
+; ".exe" to the filename created by save-system.
+
+          #+mswindows "gcl.exe"
+          #-mswindows "gcl")
+         (gcl-exec-file
+          (concatenate 'string
+                       (namestring (truename ""))
+                       gcl-exec-name
+                       "."
+                       ext+)))
+    (if write-worklispext
+        (with-open-file (str "worklispext" :direction :output)
+                        (format str ext+)))
+    (if (probe-file sysout-name)
+        (delete-file sysout-name))
+    (if (probe-file gcl-exec-file)
+        (delete-file gcl-exec-file))
+    (with-open-file (str sysout-name :direction :output)
+                    (write-exec-file str "exec ~s $*~%" gcl-exec-file))
+    (cond ((and set-optimize-maximum-pages
+                (boundp 'si::*optimize-maximum-pages*))
+
+; We follow a suggestion of Camm Maguire by setting
+; 'si::*optimize-maximum-pages* to t just before the save.  We avoid the
+; combination of 'si::*optimize-maximum-pages* and sgc-on for GCL versions
+; through 2.6.3, because of problematic interactions between SGC and
+; si::*optimize-maximum-pages*.  This issue has been fixed starting with GCL
+; 2.6.4.  Since si::*optimize-maximum-pages* is only bound starting with
+; sub-versions of 2.6, the problem only exists there.
+
+           (cond ((or (not (fboundp 'si::sgc-on))
+                      (gcl-version-> 2 6 3))
+                  (setq si::*optimize-maximum-pages* t)))))
+    (chmod-executable sysout-name)
+    (si::save-system (concatenate 'string sysout-name "." ext))))
+
 #+akcl
 (defun save-acl2-in-akcl (sysout-name gcl-exec-name
                                       &optional mode do-not-save-gcl)
   (setq *acl2-allocation-alist*
 
 ; If *acl2-allocation-alist* is rebound before allocation is done in
-; si::*top-level-hook*, e.g., if it is bound in one's init.lsp or acl2-init.lsp
-; file, then such binding will override this one.  The package name shouldn't
-; matter for the keys in user's alist, but in the code below we need to keep
-; 'hole in the ACL2 package because we refer to it below.
+; si::*top-level-hook*, e.g., if it is bound in one's init.lisp or
+; acl2-init.lsp file, then such binding will override this one.  The package
+; name shouldn't matter for the keys in user's alist, but in the code below we
+; need to keep 'hole in the ACL2 package because we refer to it below.
 
 ; Historical Comments:
 
@@ -518,9 +609,8 @@
                     (cond (mode
                            (format nil "~% Initialized with ~a." mode))
                           (t ""))
-                    (subseq *copy-of-acl2-version*
-                            (1+ (search "." *copy-of-acl2-version*))
-                            (search *copy-of-acl2-version* "(")))
+                    (eval '(latest-release-note-string)) ; avoid possible warning
+                    )
             (maybe-load-acl2-init)
             (cond
              (*acl2-allocation-alist*
@@ -546,7 +636,8 @@
 ; The following is important so that ACL2 functions are efficient in certain
 ; situations.  For example, (aref1 'foo foo n) should avoid boxing a fixnum n.
 
-  (proclaim-files)
+  #-acl2-mv-as-values (proclaim-files)
+  #+acl2-mv-as-values (load "acl2-proclaims.lisp")
 
 ; Return to normal allocation growth.  Keep this in sync with load-acl2, which
 ; had presumably already set the allocation growth to be particularly slow.
@@ -613,65 +704,52 @@
 ;    (print "Set hole size") ;debugging
     (si::set-hole-size new-hole-size))
 
+; The calculation above is legacy.  Now we increment the hole size to 20% of
+; max-pages instead of the default 10%.  Camm Maguire says that "Larger values
+; allow quick allocation of pages without triggering gc" and that the hole is
+; part of the virtual (not resident) memory size, rather than being saved to
+; disk.
+
+  (let ((new-size (floor si:*lisp-maxpages* 5)))
+    (if (< (si:get-hole-size) new-size)
+        (si::set-hole-size new-size)))
+
 ;  (print (true-listp (w *the-live-state*))) ;swap in the world's pages
 
 ;  (print "Save the system") ;debugging
   (when (not do-not-save-gcl)
-    (if (probe-file "worklispext")
-        (delete-file "worklispext"))
-    (let* ((ext "gcl")
-           (ext+
+    (save-acl2-in-akcl-aux sysout-name gcl-exec-name t t)))
 
-; We deal with the apparent fact that Windows implementations of GCL append
-; ".exe" to the filename created by save-system.
-
-            #+mswindows "gcl.exe"
-            #-mswindows "gcl")
-           (dir (namestring (truename "")))
-           (gcl-exec-file
-            (concatenate 'string
-                         dir
-                         gcl-exec-name
-                         "."
-                         ext+)))
-      (with-open-file (str "worklispext" :direction :output)
-                      (format str ext+))
-      (if (probe-file sysout-name)
-          (delete-file sysout-name))
-      (with-open-file (str sysout-name :direction :output)
-                      (format str
-                              "#!/bin/sh~%exec ~s -dir ~s~%"
-                              gcl-exec-file
-                              dir))
-      (cond ((boundp 'si::*optimize-maximum-pages*)
-
-; We follow a suggestion of Camm Maguire by setting
-; 'si::*optimize-maximum-pages* to t just before the save.  We avoid the
-; combination of 'si::*optimize-maximum-pages* and sgc-on for GCL versions
-; through 2.6.3, because of problematic interactions between SGC and
-; si::*optimize-maximum-pages*.  This issue has been fixed starting with GCL
-; 2.6.4.  Since si::*optimize-maximum-pages* is only bound starting with
-; sub-versions of 2.6, the problem only exists there.
-
-             (cond ((or (not (fboundp 'si::sgc-on))
-                        (gcl-version-> 2 6 3))
-                    (setq si::*optimize-maximum-pages* t)))))
-      (si::save-system (concatenate 'string sysout-name "." ext)))))
+#+akcl
+(defun save-exec-raw (sysout-name)
+  (setq *acl2-allocation-alist* nil) ; Don't meddle with allocations.
+  (setq *acl2-default-restart-complete* nil)
+  (save-acl2-in-akcl-aux sysout-name sysout-name nil nil))
 
 (defvar *acl2-default-restart-complete* nil)
 
 (defun acl2-default-restart ()
   (if *acl2-default-restart-complete*
       (return-from acl2-default-restart nil))
+
+  #+openmcl
+  (progn
+
+; In OpenMCL, print greeting now, rather than upon first re-entry to ACL2 loop.
+; Here we follow a suggestion from Gary Byers.
+
+    (format t "~&Welcome to ~A ~A!~%"
+            (lisp-implementation-type)
+            (lisp-implementation-version))
+    (setq ccl::*inhibit-greeting* t))
   (format t *saved-string*
           *copy-of-acl2-version*
           *saved-build-date*
           (cond (*saved-mode*
                  (format nil "~% Initialized with ~a." *saved-mode*))
                 (t ""))
-          (subseq *copy-of-acl2-version*
-                  (1+ (search "." *copy-of-acl2-version*))
-                  (search *copy-of-acl2-version* "(")))
+          (eval '(latest-release-note-string)) ; avoid possible warning
+          )
   (maybe-load-acl2-init)
   (in-package "ACL2")
 
@@ -702,6 +780,12 @@
   (acl2-default-restart)
   (lp))
 
+#+sbcl
+(defun sbcl-restart ()
+  (acl2-default-restart)
+; Use eval to avoid style-warning for undefined function LP.
+  (eval '(lp)))
+
 #+lucid
 (defun save-acl2-in-lucid (sysout-name &optional mode)
   (setq *saved-mode* mode)
@@ -718,18 +802,44 @@
 ; support (Dave Fox) has pointed out, we need to be sure to call
 ; (mp:initialize-multiprocessing) when starting up.  See LP for that call.
 
-  (setq sys:*sg-default-size* 128000)
+; But even the following wasn't adequate for (verify-guards read-utf8-fast ...)
+; in books/unicode/read-utf8.lisp:
+; (setq sys:*sg-default-size* 128000)
+; So we try the following.
+
+  (setq sys:*sg-default-size* 1024000)
+
   (system::save-image sysout-name :restart-function 'acl2-default-restart
                       :gc t))
 
+; The definition of save-exec-raw for lispworks did not work (Lispworks 4.4.5,
+; and probably Lispworks 4.2).  An attempt resulted in the following error:
+
+;   ACL2 5 > (save-exec "my-lw" "Changes made")
+
+;   Error: You cannot save an image while multiprocessing or the environment is running.
+;   Place the call to HARLEQUIN-COMMON-LISP:SAVE-IMAGE in a file and use the -init command line option.
+
+; If it becomes important to save an ACL2 image for lispworks from an ACL2
+; session, we can figure this out at that time (and perhaps get some help from
+; the user, who may be knowledgeable about lispworks).
+
+; #+lispworks
+; (defun save-exec-raw (sysout-name)
+;   (setq *acl2-default-restart-complete* nil)
+;   (system::save-image sysout-name :restart-function 'acl2-default-restart
+;                       :gc t))
+
+#+lispworks
+(defun save-exec-raw (sysout-name)
+  (declare (ignore sysout-name))
+  (error "Save-exec-raw is not implemented in lispworks.  Feel free to contact~%~
+          the implementors, and feel free to read the comment above the~%~
+          definition  of save-exec-raw for #+lispworks in ACL2 source file~%~
+          acl2-init.lisp."))
+
 #+cmu
-(defun save-acl2-in-cmulisp (sysout-name &optional mode core-name)
-  (setq *saved-mode* mode)
-
-; Keep the following code in sync with similar code in save-acl2-in-allegro.
-; We probably should break out this common code into a subroutine used by both
-; functions.
-
+(defun save-acl2-in-cmulisp-aux (sysout-name core-name)
   (let ((eventual-sysout-core
          (concatenate 'string
                       (namestring (truename ""))
@@ -737,12 +847,10 @@
                       ".core"))
         (sysout-core
          (concatenate 'string sysout-name ".core")))
-    (if (probe-file "worklispext")
-        (delete-file "worklispext"))
-    (with-open-file (str "worklispext" :direction :output)
-                    (format str "core"))
     (if (probe-file sysout-name)
         (delete-file sysout-name))
+    (if (probe-file eventual-sysout-core)
+        (delete-file eventual-sysout-core))
     (with-open-file ; write to nsaved_acl2
      (str sysout-name :direction :output)
      (let* ((prog0 (car extensions::*command-line-strings*))
@@ -762,10 +870,12 @@
                          ((equal (subseq prog0 (- len 3) len) "bin")
                           (concatenate 'string prog0 "/lisp"))
                          (t prog0))))
-       (format str
-               "#!/bin/sh~%~s -core ~s -eval '(acl2::cmulisp-restart)'~%"
-               prog1
-               eventual-sysout-core)))
+       (write-exec-file str
+                        "~s -core ~s -eval '(acl2::cmulisp-restart)' $*~%"
+                        prog1
+                        eventual-sysout-core)))
+    (chmod-executable sysout-name)
+    (system::gc)
     (extensions::save-lisp sysout-core :load-init-file nil :site-init nil
 
 ; We call print-herald in cmulisp-restart, so that the herald is printed
@@ -773,24 +883,77 @@
 
                            :print-herald nil)))
 
+#+cmu
+(defun save-acl2-in-cmulisp (sysout-name &optional mode core-name)
+  (setq *saved-mode* mode)
+  (if (probe-file "worklispext")
+      (delete-file "worklispext"))
+  (with-open-file (str "worklispext" :direction :output)
+                  (format str "core"))
+  (save-acl2-in-cmulisp-aux sysout-name core-name))
+
+#+cmu
+(defun save-exec-raw (sysout-name)
+  (setq *acl2-default-restart-complete* nil)
+  (save-acl2-in-cmulisp-aux sysout-name sysout-name))
+
+#+sbcl
+(defun save-acl2-in-sbcl-aux (sysout-name core-name)
+  (declaim (optimize (sb-ext:inhibit-warnings 3)))
+  (let ((eventual-sysout-core
+         (concatenate 'string
+                      (namestring (truename ""))
+                      core-name
+                      ".core"))
+        (sysout-core
+         (concatenate 'string sysout-name ".core")))
+    (if (probe-file sysout-name)
+        (delete-file sysout-name))
+    (if (probe-file eventual-sysout-core)
+        (delete-file eventual-sysout-core))
+    (with-open-file ; write to nsaved_acl2
+     (str sysout-name :direction :output)
+     (let* ((prog (car sb-ext:*posix-argv*)))
+       (write-exec-file
+        str
+        "~s --core ~s --userinit /dev/null --eval '(acl2::sbcl-restart)'"
+        prog
+        eventual-sysout-core)))
+    (chmod-executable sysout-name)
+    ;; In SBCL 0.9.3 the read-only space is too small for dumping ACL2 on x86,
+    ;; so we have to specify :PURIFY NIL. This will unfortunately result in
+    ;; some core file bloat, and slightly slower startup.
+    (sb-ext:gc)
+    (sb-ext:save-lisp-and-die sysout-core
+                              :purify 
+                              #+(or x86 x86-64) nil
+                              #-(or x86 x86-64) t)))
+                           
+#+sbcl
+(defun save-acl2-in-sbcl (sysout-name &optional mode core-name)
+  (with-warnings-suppressed
+   (setq *saved-mode* mode)
+   (if (probe-file "worklispext")
+       (delete-file "worklispext"))
+   (with-open-file (str "worklispext" :direction :output)
+                   (format str "core"))
+   (save-acl2-in-sbcl-aux sysout-name core-name)))
+
+#+sbcl
+(defun save-exec-raw (sysout-name)
+  (with-warnings-suppressed
+   (setq *acl2-default-restart-complete* nil)
+   (save-acl2-in-sbcl-aux sysout-name sysout-name)))
+
 #+allegro
-(defun save-acl2-in-allegro (sysout-name &optional mode dxl-name)
-
-; Note that dxl-name should, if supplied, be a relative pathname string, not
-; absolute.
-
+(defun save-acl2-in-allegro-aux (sysout-name dxl-name)
+  (excl:gc t) ; Suggestions are welcome for better gc call(s)!
   #+(and allegro-version>= (version>= 4 3))
   (progn
-    (setq *saved-mode* mode)
     (tpl:setq-default *PACKAGE* (find-package "ACL2"))
     (setq EXCL:*RESTART-INIT-FUNCTION* 'acl2-default-restart)
     #+(and allegro-version>= (version>= 5 0))
     (progn
-      (if (probe-file "worklispext")
-          (delete-file "worklispext"))
-      (with-open-file (str "worklispext" :direction :output)
-                      (format str "dxl"))
-      (load "allegro-acl2-trace.lisp") ; Robert Krug's trace patch
 
 ; Allegro 5.0 and later no longer supports standalone images.  Instead, one
 ; creates a .dxl file using dumplisp and then starts up ACL2 using the original
@@ -830,10 +993,10 @@
              (sysout-dxl
               (concatenate 'string sysout-name ".dxl")))
         (write-acl2rc save-dir)
-        (excl:dumplisp :name sysout-dxl)
         (with-open-file ; write to nsaved_acl2
          (str sysout-name :direction :output)
-         (format str
+         (write-exec-file
+          str
 
 ; We use ~s instead of ~a below because John Cowles has told us that in Windows
 ; 98, the string quotes seem necessary for the first string and desirable for
@@ -843,19 +1006,39 @@
 ; go with format arg (rc-filename save-dir), because we know of no way in
 ; Allegro 6.2 to avoid getting Allegro copyright information printed upon :q if
 ; we start up in the ACL2 read-eval-print loop.
-#|               "#!/bin/sh~%~s -I ~s -L ~s~%" |#
+          #|               "~s -I ~s -L ~s~%" |#
 
-                 "#!/bin/sh~%~s -I ~s~%"
-                 (system::command-line-argument 0)
-                 eventual-sysout-dxl))))
+          "~s -I ~s $*~%"
+          (system::command-line-argument 0)
+          eventual-sysout-dxl))
+        (chmod-executable sysout-name)
+        (excl:dumplisp :name sysout-dxl)))
     #-(and allegro-version>= (version>= 5 0))
     (excl:dumplisp :name sysout-name :checkpoint nil))
   #-(and allegro-version>= (version>= 4 3))
   (progn
-    (setq *saved-mode* mode)
     (excl:dumplisp :name sysout-name :checkpoint t
                    :restart-function 'acl2-default-restart)))
 
+#+allegro
+(defun save-acl2-in-allegro (sysout-name &optional mode dxl-name)
+
+; Note that dxl-name should, if supplied, be a relative pathname string, not
+; absolute.
+
+  (setq *saved-mode* mode)
+  (if (probe-file "worklispext")
+      (delete-file "worklispext"))
+  (with-open-file (str "worklispext" :direction :output)
+                  (format str "dxl"))
+  (load "allegro-acl2-trace.lisp") ; Robert Krug's trace patch
+  (save-acl2-in-allegro-aux sysout-name dxl-name))
+
+#+allegro
+(defun save-exec-raw (sysout-name)
+  (setq *acl2-default-restart-complete* nil)
+  (save-acl2-in-allegro-aux sysout-name sysout-name))
+
 (defun rc-filename (dir)
   (concatenate 'string dir ".acl2rc"))
 
@@ -878,37 +1061,40 @@
                   (ACL2::LP)~%")))))
 
 #+clisp
-(defun save-acl2-in-clisp (sysout-name &optional mode mem-name)
-  (setq *saved-mode* mode)
-
-; Keep the following code in sync with similar code in save-acl2-in-allegro.
-; We probably should break out this common code into a subroutine used by both
-; functions.
-
+(defun save-acl2-in-clisp-aux (sysout-name mem-name)
   (let* ((save-dir (namestring (truename "")))
          (eventual-sysout-mem
           (concatenate 'string save-dir mem-name ".mem"))
          (sysout-mem
           (concatenate 'string sysout-name ".mem")))
-    (if (probe-file "worklispext")
-        (delete-file "worklispext"))
-    (with-open-file (str "worklispext" :direction :output)
-                    (format str "mem"))
     (if (probe-file sysout-name)
         (delete-file sysout-name))
+    (if (probe-file eventual-sysout-mem)
+        (delete-file eventual-sysout-mem))
     (write-acl2rc save-dir)
     (with-open-file ; write to nsaved_acl2
      (str sysout-name :direction :output)
-     (format str
+     (write-exec-file str
 
 ; We take Noah Friedman's suggestion of using "exec" since there is no reason
 ; to keep the saved_acl2 shell script in the process table.
 
-             "#!/bin/sh~%exec ~s -i ~s -p ACL2 -M ~s~%"
-             (or (ext:getenv "LISP") "clisp")
-             (rc-filename save-dir)
-             eventual-sysout-mem))
+                      "exec ~s -i ~s -p ACL2 -M ~s -m ~sMB $*~%"
+                      (or (ext:getenv "LISP") "clisp")
+                      (rc-filename save-dir)
+                      eventual-sysout-mem
+
+; Here we choose a value of 10 for the -m argument.  We have found that without
+; setting -m, we get a stack overflow in books/unicode/read-utf8.lisp at
+; (verify-guards read-utf8-fast ...) when running on CLISP 2.34 on Linux.
+; CLISP documentation at http://clisp.cons.org/clisp.html#opt-memsize says that
+; it is "common to specify 10 MB" for the value of -m; since that suffices to
+; eliminate the stack overflow mentioned above, we use that value.
+
+                      10))
+    (chmod-executable sysout-name)
     (in-package "ACL2")
+    (ext:gc)
     (ext:saveinitmem sysout-mem
                      :quiet nil
 
@@ -918,17 +1104,33 @@
 
                      :init-function 'acl2-default-restart)))
 
-#+openmcl
-(defun save-acl2-in-openmcl (sysout-name &optional mode core-name)
+#+clisp
+(defun save-acl2-in-clisp (sysout-name &optional mode mem-name)
   (setq *saved-mode* mode)
-  (in-package "ACL2")
-  (proclaim-files)
-  (load "openmcl-acl2-trace.lisp")
-  (let ((core-name (concatenate 'string
-				(namestring (truename ""))
-				"/"
-				core-name
-				".dppccl")))
+  (if (probe-file "worklispext")
+      (delete-file "worklispext"))
+  (with-open-file (str "worklispext" :direction :output)
+                  (format str "mem"))
+  (save-acl2-in-clisp-aux sysout-name mem-name))
+
+#+clisp
+(defun save-exec-raw (sysout-name)
+  (setq *acl2-default-restart-complete* nil)
+  (save-acl2-in-clisp-aux sysout-name sysout-name))
+
+#+openmcl
+(defun save-acl2-in-openmcl-aux (sysout-name core-name)
+  (let* ((current-dir0 (namestring (truename "")))
+         (current-dir (if (eql (char current-dir0 (1- (length current-dir0)))
+                               #\/)
+                          current-dir0
+                        (concatenate 'string
+                                     current-dir0
+                                     "/")))
+         (core-name (concatenate 'string
+                                 current-dir
+                                 core-name
+                                 ".dppccl")))
     (when (probe-file sysout-name)
 
 ; At one point we supplied :if-exists :overwrite in the first argument to
@@ -937,14 +1139,39 @@
 ; attempting to write to it.
 
       (delete-file sysout-name))
+    (when (probe-file core-name)
+      (delete-file core-name))
     (with-open-file ; write to nsaved_acl2
-      (str sysout-name :direction :output)
-      (format str
-	      "#!/bin/sh~%~s -I ~s -e \"(acl2::acl2-default-restart)\""
-	      (or (car ccl::*command-line-argument-list*) "openmcl")
-	      core-name))
+     (str sysout-name :direction :output)
+     (write-exec-file str
+                      "~a~%~s -I ~s -e \"(acl2::acl2-default-restart)\" $*~%"
+                      (let ((default-dir
+                              (ccl::getenv "CCL_DEFAULT_DIRECTORY")))
+                        (if default-dir
+                            (format nil
+                                    "export CCL_DEFAULT_DIRECTORY=~s"
+                                    default-dir)
+                          ""))
+                      (or (car ccl::*command-line-argument-list*) "openmcl")
+                      core-name))
+    (chmod-executable sysout-name)
+    (ccl::gc)
     (ccl:save-application core-name)))
 
+#+openmcl
+(defun save-acl2-in-openmcl (sysout-name &optional mode core-name)
+  (setq *saved-mode* mode)
+  (in-package "ACL2")
+  #-acl2-mv-as-values (proclaim-files)
+  #+acl2-mv-as-values (load "acl2-proclaims.lisp")
+  (load "openmcl-acl2-trace.lisp")
+  (save-acl2-in-openmcl-aux sysout-name core-name))
+
+#+openmcl
+(defun save-exec-raw (sysout-name)
+  (setq *acl2-default-restart-complete* nil)
+  (save-acl2-in-openmcl-aux sysout-name sysout-name))
+
 ; Since saved-build-date-string is avoided for MCL, we avoid the following too
 ; (which is not applicable to MCL sessions anyhow).
 #-(and mcl (not openmcl))
@@ -956,7 +1183,7 @@
 
                             do-not-save-gcl)
   #-akcl (declare (ignore do-not-save-gcl))
-  #-(or akcl allegro cmu clisp openmcl)
+  #-(or akcl allegro cmu sbcl clisp openmcl)
   (declare (ignore other-info))
 
   #+akcl
@@ -998,22 +1225,56 @@
   (save-acl2-in-allegro "nsaved_acl2" mode other-info)
   #+cmu
   (save-acl2-in-cmulisp "nsaved_acl2" mode other-info)
+  #+sbcl
+  (save-acl2-in-sbcl "nsaved_acl2" mode other-info)
   #+clisp
   (save-acl2-in-clisp "nsaved_acl2" mode other-info)
   #+openmcl
   (save-acl2-in-openmcl "nsaved_acl2" mode other-info)
-  #-(or akcl lucid lispworks allegro clisp openmcl)
+  #-(or akcl lucid lispworks allegro clisp openmcl cmu sbcl)
   (error "We do not know how to save ACL2 in this Common Lisp.")
   (format t "Saving of ACL2 is complete.~%"))
 
-(defun proclaim-files ()
+(defun proclaim-files (&optional outfilename)
 
 ; IMPORTANT:  This function assumes that the defconst forms in the
 ; given files have already been evaluated.  One way to achieve this
 ; state of affairs, of course, is to load the files first.
 
-  (dolist (fl *copy-of-acl2-files*)
-          (proclaim-file (format nil "~a.lisp" fl))))
+  (if outfilename
+      (format t
+              "Writing proclaim forms for ACL2 source files to file ~s.~%"
+              outfilename)
+    (format t
+              "Generating and evaluating proclaim forms for ACL2 source ~
+               files.~%"))
+  (let (str)
+    (when outfilename
+      (if (probe-file outfilename)
+          (delete-file outfilename))
+      (or (setq str (open outfilename :direction :output))
+          (error "Unable to open file ~s for output." outfilename))
+      (format str "(in-package \"ACL2\")~%"))
+    (dolist (fl *copy-of-acl2-files*)
+      (proclaim-file (format nil "~a.lisp" fl) str))))
+
+(defun generate-acl2-proclaims ()
+  (let ((filename "acl2-proclaims.lisp"))
+    #-(and acl2-mv-as-values (or gcl openmcl))
+    (progn
+      (if (probe-file filename)
+          (delete-file filename))
+      (with-open-file
+       (str filename :direction :output)
+       (format str "(in-package \"ACL2\")~%~%")
+       (format str "; No proclaims are generated here for this host Lisp.~%"))
+      (return-from generate-acl2-proclaims nil))
+    (format t "Beginning load-acl2 and initialize-acl2 on behalf of ~
+               generate-acl2-proclaims.~%")
+    (load-acl2 t)
+; Use funcall to avoid compiler warning in (at least) OpenMCL.
+    (funcall 'initialize-acl2 'include-book nil nil t)
+    (proclaim-files filename)))
 
 (defun acl2 nil
   (let ((*readtable* *acl2-readtable*))
