@@ -1,5 +1,5 @@
-; ACL2 Version 2.9.3 -- A Computational Logic for Applicative Common Lisp
-; Copyright (C) 2005  University of Texas at Austin
+; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
 ; (C) 1997 Computational Logic, Inc.  See the documentation topic
@@ -46,8 +46,9 @@
 ; non-CLTL, assumptions that we make about Common Lisps in which ACL2 is to
 ; run.
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                               FILES
-
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; This is the file acl2.lisp, the first source file for ACL2.  The
 ; names of the other ACL2 source files are listed under *acl2-files*
@@ -61,8 +62,14 @@
 ; file axioms.lisp), ACL2 is written in the applicative Common Lisp
 ; for which ACL2 is a verification system.
 
-
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                          LISP BUGS AND QUIRKS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+; We do not yet support GCL ANSI.
+#+(and gcl ansi-cl)
+(error
+ "FATAL ERROR: ACL2 does not yet support GCL ANSI.  Please use a non-ANSI GCL.")
 
 ; To use ACL2 under Lispworks 3.2.0, execute the following to work around a
 ; bug.
@@ -100,16 +107,21 @@
 
 ; Turn off compiler verbosity going to error stream, since even >& does not
 ; seem to redirect that stream to a file.
-#+cmu
+#+(or cmu sbcl)
 (setq *compile-verbose* nil)
-#+cmu
+#+(or cmu sbcl)
 (setq *compile-print* nil)
 
 ; Turn off gc verbosity (same reason as above).
 #+cmu
 (setq ext:*gc-verbose* nil)
 
+; See later in this file for with-warnings-suppressed (after we introduce and
+; enter the ACL2 package).
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            CLTL1/CLTL2
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; For the most part, we program in a subset both of CLTL1 and CLTL2.
 ; However, there are a few places, notably the name of the main
@@ -201,8 +213,9 @@
 
 ; (5) We made a few fixes and cleanups in order to avoid compiler warnings.
 
-
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                              PACKAGES
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; We never intend that this file should be compiled, and hence we do
 ; not need to obey the CLTL1 strictures about putting all IN-PACKAGE
@@ -436,7 +449,7 @@
    (setq acl2::*copy-of-acl2-version*
 ;  Keep this in sync with the value of acl2-version in *initial-global-table*.
          (concatenate 'string
-                      "ACL2 Version 2.9.3"
+                      "ACL2 Version 3.0"
                       #+non-standard-analysis
                       "(r)"
                       #+(and mcl (not openmcl))
@@ -529,8 +542,32 @@
     (push :akcl-set-mv *features*)
   (error "Use a version of ACKL after 206"))
 
-
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                                CHECKS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+; We need the following macro in order to turn off style warnings in sbcl.
+; Perhaps we will find it useful in other lisps as well.  It is defined here
+; instead of the "LISP BUGS AND QUIRKS" section because we want to define this
+; in the ACL2 package.
+(defmacro with-warnings-suppressed (&rest forms)
+
+; Keep this in sync with the handler-bind call in init.lisp.  Thanks to Juho
+; Snellman for the SBCL form.
+
+  #+sbcl
+  `(handler-bind
+    (#+ansi-cl
+     (style-warning (lambda (c)
+                      (declare (ignore c))
+                      (invoke-restart 'muffle-warning))))
+    ,@forms)
+  #+cmucl
+  `(progn (setq ext:*gc-verbose* nil) ,@forms)
+  #+lispworks
+  `(let ((compiler::*redefinition-action* :QUIET)) ,@forms)
+  #-(or cmucl sbcl lispworks)
+  (if (cdr forms) `(progn ,@forms) (car forms)))
 
 (defconstant acl2::*acl2-status-file*
   (make-pathname :name "acl2-status"
@@ -541,18 +578,16 @@
 ; As of version 18a, cmulisp spews gc messages to the terminal even when
 ; standard and error output are redirected.  So we turn them off.
 
-  #+cmu
-  (setq extensions::*gc-verbose* nil)
-
-  (progn (or (not (probe-file *acl2-status-file*))
-             (delete-file *acl2-status-file*))
-         (load "acl2-check.lisp")
-         (with-open-file (str *acl2-status-file*
-                              :direction :output)
-                         (format str
-                                 "~s"
-                                 :checked))
-         t))
+  (with-warnings-suppressed
+   (or (not (probe-file *acl2-status-file*))
+       (delete-file *acl2-status-file*))
+   (load "acl2-check.lisp")
+   (with-open-file (str *acl2-status-file*
+                        :direction :output)
+                   (format str
+                           "~s"
+                           :checked))
+   t))
 
 (defun note-compile-ok ()
   (progn (or (not (probe-file *acl2-status-file*))
@@ -563,8 +598,9 @@
                                  "~s"
                                  :compiled))))
 
-
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                       COMPILING and LOADING, PART 1
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; To compile ACL2, invoke (load "acl2.lisp") and then invoke
 ; (acl2::compile-acl2).  Having compiled ACL2, to build ACL2, invoke
@@ -584,50 +620,24 @@
 (defvar *lisp-extension* "lisp")
 
 (defparameter *compiled-file-extension*
-  #+akcl "o"
-  #+kcl  "o"
-  #+(and lucid sparc) "sbin"
-  #+(and lucid mc68000) "lbin"
-  #+allegro "fasl"
-  #+lispworks "ufsl" ; was "wfasl" for 3.2.0
-  #+(and mcl (not openmcl) (not ppc-target)) "fasl"
-  #+(and mcl (not openmcl) ppc-target) "pfsl"
-  #+(and openmcl (not ppc-target)) "fasl"  ;; from doc, Not tested
-  #+(and openmcl ppc-target) "dfsl"
-  #+symbolics "bin"
-  #+(and cmu sparc) "sparcf"
-  #+(and cmu alpha) "axpf"
-  #+(and cmu x86) "x86f"
-  #+(and cmu ppc) "ppcf" ; thanks to Fabricio Chalub Barbosa do Rosario
-  #+clisp "fas"
-  #-(or akcl kcl lucid allegro lispworks mcl symbolics cmu clisp)
-  (do ((name
-        (progn
-          (format
-           t
-           "~%~%We are trying to initialize the parameter that specifies the~%~
-            file extension name used by the local compiler.  For example, in~%~
-            Allegro Common Lisp, the extension is \"fasl\" while in Lucid~%~
-            it is \"lbin\" or \"sbin\".  Files with such extensions are understood~%~
-            to contain compiled code.  We do not recognize this Common Lisp~%~
-            from its *features*.  Please type as a STRINGP suitable for READ~%~
-            its compiled file extension name:  ")
-          (read t))
-        (progn
-          (format
-           t
-           "~%~%It has to READ as a STRINGP!  Put quotation marks ~
-            around it:  ")
-          (read t))))
-      ((stringp name) name)))
 
-; The user is welcome to modify the following proclaim form.
+; Note that books/Makefile-generic deals with compiled file extensions as well
+; in its "clean" target.
+; Thanks to Gary Byers for suggested the following approach for #+ansi-cl,
+; which however appears to work for all Lisps supported as of early 2006.
 
-(proclaim (quote (optimize #+(or cmu lucid lispworks) (compilation-speed 0)
-                           ;; (debug 0) ; could consider for Lispworks, Allegro
-                           #+cmu (extensions:inhibit-warnings 3)
-                           (speed 3)
-                           (space #+cmu 1 #-cmu 0)
+  (pathname-type (compile-file-pathname "foo.lisp")))
+
+; The user is welcome to modify the following declaim form.
+
+(declaim (optimize #+(or cmu sbcl lucid lispworks) (compilation-speed 0)
+; The following may allow more tail recursion elimination (from "Lisp
+; Knowledgebase" at lispworks.com); might consider for Allegro CL too.
+                   #+lispworks (debug 0)
+                   #+cmu (extensions:inhibit-warnings 3)
+                   #+sbcl (sb-ext:inhibit-warnings 3)
+                   (speed 3)
+                   (space #+cmu 1 #-cmu 0) ; no longer necessary?
 
 ; WARNING:  Do not proclaim (cl-user::fixnum-safety 0) for Lispworks.  Any
 ; fixnum-safety less than 3 expects all integers to be fixnums!
@@ -637,7 +647,10 @@
 ; bitn-lam0 from books/rtl/rel2/support/lop3.lisp.  (See save-acl2-in-lispworks
 ; for how we have eliminated that stack overflow.)
 
-                           (safety 0))))
+                   (safety 0)))
+
+#+sbcl ; turn off compiler notes (noisy)
+(declaim (sb-ext:muffle-conditions sb-ext:compiler-note))
 
 #+allegro
 (eval (read-from-string "
@@ -673,8 +686,9 @@
   #-(or cltl2 cmu)
   form)
 
-
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                           BASIC FUNCTIONS TO BE COMPILED
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;  Functions for proclaiming and for defining ACL2's Implementation of the
 ;  Backquote Readmacro are to be compiled, so we do not include them in the
@@ -688,11 +702,12 @@
 (defconstant *acl2-read-character-terminators*
   '(#\Tab #\Newline #\Page #\Space #\" #\' #\( #\) #\; #\` #\,))
 
-(let ((object-file (make-pathname :name "acl2-fns"
-                                  :type *compiled-file-extension*)))
+(our-with-compilation-unit
+ (let ((object-file (make-pathname :name "acl2-fns"
+                                   :type *compiled-file-extension*)))
 
 ; At one time we avoided recompiling and had the following code inside a COND:
-#|
+   #|
    ((and (probe-file object-file)
          (<= (file-write-date "acl2-fns.lisp")
              (file-write-date object-file)))
@@ -702,16 +717,17 @@
 ; the system with Allegro under Linux, the same ".fasl" extension could fool us
 ; into thinking that recompilation is not necessary.
 
-  (load "acl2-fns.lisp") ;we like to load before compiling
-  (compile-file "acl2-fns.lisp")
+   (load "acl2-fns.lisp") ;we like to load before compiling
+   (compile-file "acl2-fns.lisp")
 
 ; Note that load-compiled is not used below, but on the other hand we are still
 ; using the original readtable here so that's not a problem.
 
-  (load object-file))
-
+   (load object-file)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                           ACL2-READTABLE
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defparameter *acl2-readtable*
   (copy-readtable nil)
@@ -1061,16 +1077,28 @@ implementations:
 
 Newline
 
-|#
+|# ; |
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                       COMPILING and LOADING, PART 2
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun compile-acl2 (&optional maybe-recompile)
+(defun compile-acl2 (&optional use-acl2-proclaims)
+
+  (with-warnings-suppressed
+
+   #+sbcl
+   (declaim (optimize (safety 0) (space 0) (speed 3) (debug 0)))
 
 ; Here is a natural place to put compiler options.  In fact, we put
 ; them above, globally.
 
-; (proclaim '(optimize (safety 0) (space 0) (speed 3)))
+; (declaim (optimize (safety 0) (space 0) (speed 3)))
+
+; However, Juho Snellman points out that SBCL resets the compiler policy on
+; entry to LOAD / COMPILE-FILE, and restores the old value once the file has
+; been loaded / compiled; thus the global declaim is no longer in effect once
+; COMPILE-ACL2 gets called.
 
 ; Note on Illegal Instructions:  If ACL2 causes an illegal instruction
 ; trap it is difficult to figure out what is happening.  Here is a
@@ -1078,8 +1106,8 @@ Newline
 ; (0) create an events file or book that will recreate the
 ;     the state up to the event that causes the illegal instruction.
 ; (1) Copy saved_acl2 to old_saved_acl2
-; (2) replace the proclaim above with
-;     (proclaim '(optimize (safety 3) (space 0) (speed 0)));
+; (2) replace the declaim above with
+;     (declaim (optimize (safety 3) (space 0) (speed 0)));
 ; (3) make full init
 ;     this will fail because of stack overflows during booting.
 ; (4) fire up old_saved_acl2
@@ -1098,7 +1126,7 @@ Newline
 ;            t)
 ;(10) (si::use-fast-links nil)
 ;(11) enter (LP) and execute the offending event.
-;(12) once the bug is fixed, be sure to change the proclaim
+;(12) once the bug is fixed, be sure to change the declaim
 ;     back to (safety 0) (speed 3) before recompiling.
 
 ; Note on loading before compiling.  We load each ACL2 source file
@@ -1111,48 +1139,49 @@ Newline
 ; In general, and for the same reason, all ACL2 user checked files
 ; are also be loaded before they are compiled.
 
+   #-acl2-mv-as-values
+   (when use-acl2-proclaims
+     (return-from compile-acl2 nil))
+
 ; As of version 18a, cmulisp spews gc messages to the terminal even when
 ; standard and error output are redirected.  So we turn them off.
 
-  #-acl2-mv-as-values
-  (when maybe-recompile
-    (return-from compile-acl2 nil))
-
-  #+cmu
-  (setq extensions::*gc-verbose* nil)
-
-  (cond
-   ((or (not (probe-file *acl2-status-file*))
-        (with-open-file (str *acl2-status-file*
-                             :direction :input)
-                        (not (eq (read str nil)
-                                 :checked))))
-    (check-suitability-for-acl2)))
-  (our-with-compilation-unit
-   (let ((*readtable* *acl2-readtable*)
-         #+akcl
+   (cond
+    ((or (not (probe-file *acl2-status-file*))
+         (with-open-file (str *acl2-status-file*
+                              :direction :input)
+                         (not (eq (read str nil)
+                                  :checked))))
+     (check-suitability-for-acl2)))
+   (our-with-compilation-unit
+    (let ((*readtable* *acl2-readtable*)
+          #+akcl
 
 ; AKCL compiler note stuff.  We have so many tail recursive functions
 ; that the notes about tail recursion optimization are just too much
 ; to take.
 
-         (compiler:*suppress-compiler-notes* t))
-     (when maybe-recompile
-       (load "acl2-proclaims.lisp"))
-     (dolist (name *copy-of-acl2-files*)
-             (or (equal name "defpkgs")
-                 (let ((source (make-pathname :name name
-                                              :type *lisp-extension*)))
-                   (load source)
-                   (or (equal name "proof-checker-pkg")
-                       (progn
-                         (proclaim-file source) ; no-op before initialize-acl2
-                         (compile-file source)
-                         (load-compiled
-                          (make-pathname :name name
-                                         :type *compiled-file-extension*)))))))
-     (note-compile-ok)
-     )))
+          (compiler:*suppress-compiler-notes* t))
+      (when use-acl2-proclaims
+        (load "acl2-proclaims.lisp"))
+      (dolist (name *copy-of-acl2-files*)
+        (or (equal name "defpkgs")
+            (let ((source (make-pathname :name name
+                                         :type *lisp-extension*)))
+              (load source)
+              (or (equal name "proof-checker-pkg")
+                  (progn
+                    (when (not use-acl2-proclaims)
+
+; So, we have not loaded acl2-proclaims.lisp.
+
+                      (proclaim-file source))
+                    (compile-file source)
+                    (load-compiled
+                     (make-pathname :name name
+                                    :type *compiled-file-extension*)))))))
+      (note-compile-ok)
+      ))))
 
 (defun no-object-file-or-out-of-date-object-file (fl)
   (or (null (probe-file
@@ -1164,60 +1193,62 @@ Newline
 
 (defun quick-compile-acl2 (&optional very-fast use-acl2-proclaims)
 
+  (with-warnings-suppressed
+
 ; Here is a natural place to put compiler options.
 
-; (proclaim '(optimize (safety 0) (space 0) (speed 3)))
+; (declaim (optimize (safety 0) (space 0) (speed 3)))
 
 ; As of version 18a, cmulisp spews gc messages to the terminal even when
 ; standard and error output are redirected.  So we turn them off.
 
-  #+cmu
-  (setq extensions::*gc-verbose* nil)
+   #+cmu
+   (setq extensions::*gc-verbose* nil)
 
-  (cond
-   ((or (not (probe-file (make-pathname :name "acl2-status"
-                                        :type "txt")))
-        (with-open-file (str (make-pathname :name "acl2-status"
-                                            :type "txt")
-                             :direction :input)
-                        (not (eq (read str nil)
-                                 :checked))))
-    (check-suitability-for-acl2)))
-  (our-with-compilation-unit
-   (let ((compile-rest-flg nil)
-         (*readtable* *acl2-readtable*)
-         #+akcl
-         (si:*notify-gbc* nil)
-         #+akcl
+   (cond
+    ((or (not (probe-file (make-pathname :name "acl2-status"
+                                         :type "txt")))
+         (with-open-file (str (make-pathname :name "acl2-status"
+                                             :type "txt")
+                              :direction :input)
+                         (not (eq (read str nil)
+                                  :checked))))
+     (check-suitability-for-acl2)))
+   (our-with-compilation-unit
+    (let ((compile-rest-flg nil)
+          (*readtable* *acl2-readtable*)
+          #+akcl
+          (si:*notify-gbc* nil)
+          #+akcl
 
 ; AKCL compiler note stuff.  We have so many tail recursive functions
 ; that the notes about tail recursion optimization are just too much
 ; to take.
 
-         (compiler:*suppress-compiler-notes* t)
-         (files (remove "defpkgs" *copy-of-acl2-files* :test #'equal)))
-     (cond
-      ((some #'no-object-file-or-out-of-date-object-file files)
-       (when use-acl2-proclaims
-         (load "acl2-proclaims.lisp"))
-       (dolist
-        (fl files)
-        (let ((source (make-pathname :name fl :type *lisp-extension*))
-              (object (make-pathname :name fl :type *compiled-file-extension*)))
-          (cond
-           ((equal fl "proof-checker-pkg")
-            (load source))
-           ((or compile-rest-flg (no-object-file-or-out-of-date-object-file fl))
-            (load source)
-            (proclaim-file source)
-            (when (not very-fast)
-                  (setq compile-rest-flg t))
-            (compile-file source)
-            (load-compiled object))
-           (t (load-compiled object)
-              (proclaim-file source))))))
-      (t "Nothing to do."))))
-  (note-compile-ok))
+          (compiler:*suppress-compiler-notes* t)
+          (files (remove "defpkgs" *copy-of-acl2-files* :test #'equal)))
+      (cond
+       ((some #'no-object-file-or-out-of-date-object-file files)
+        (when use-acl2-proclaims
+          (load "acl2-proclaims.lisp"))
+        (dolist
+          (fl files)
+          (let ((source (make-pathname :name fl :type *lisp-extension*))
+                (object (make-pathname :name fl :type *compiled-file-extension*)))
+            (cond
+             ((equal fl "proof-checker-pkg")
+              (load source))
+             ((or compile-rest-flg (no-object-file-or-out-of-date-object-file fl))
+              (load source)
+              (proclaim-file source)
+              (when (not very-fast)
+                (setq compile-rest-flg t))
+              (compile-file source)
+              (load-compiled object))
+             (t (load-compiled object)
+                (proclaim-file source))))))
+       (t "Nothing to do."))))
+   (note-compile-ok)))
 
 #+gcl
 (defvar user::*fast-acl2-gcl-build* nil)
@@ -1226,78 +1257,69 @@ Newline
 
   #-akcl (declare (ignore fast)) ; fast only avoids slow growth during gcl init
 
-; As of version 18a, cmulisp spews gc messages to the terminal even when
-; standard and error output are redirected.  So we turn them off.
-
-  #+cmu
-  (setq extensions::*gc-verbose* nil)
+  (with-warnings-suppressed
 
 ; If we are in the first pass of two passes because of acl2-mv-as-values, then
 ; don't waste time doing the slow build for GCL (where we compile all *1*
 ; functions as we go through initialization).
 
-  #+(and gcl acl2-mv-as-values)
-  (when fast
-    (setq user::*fast-acl2-gcl-build* t))
+   #+(and gcl acl2-mv-as-values)
+   (when fast
+     (setq user::*fast-acl2-gcl-build* t))
 
-  #+akcl
+   #+akcl
 
 ; We grow the image slowly, since we now do allocation on start-up.  We are
 ; assuming that people will be using load-acl2 only as part of the process of
 ; building a saved image, and hence that this slow growth policy will be undone
 ; by the function save-acl2-in-akcl.  If we are 
 
-  (when (not fast)
-    (sloop::sloop
-     for type in
-     '(cons fixnum symbol array string cfun sfun
+   (when (not fast)
+     (sloop::sloop
+      for type in
+      '(cons fixnum symbol array string cfun sfun
 
 ; In akcl, at least some versions of it, we cannot call allocate-growth on the
 ; following two types.
 
-            #+gcl contiguous
-            #+gcl relocatable
-            )
-     do
-     (cond
-      ((or (boundp 'si::*gcl-major-version*) ;GCL 2.0 or greater
-           (and (boundp 'si::*gcl-version*) ;GCL 1.1
-                (= si::*gcl-version* 1)))
-       (si::allocate-growth type 1 10 50 2))
-      (t (si::allocate-growth type 1 10 50)))))
-  (cond
-   ((or (not (probe-file *acl2-status-file*))
-        (with-open-file (str *acl2-status-file*
-                             :direction :input)
-                        (not (member (read str nil)
-                                     '(:compiled :initialized)))))
-    (error "Please compile ACL2 using ~s or~%~
+             #+gcl contiguous
+             #+gcl relocatable
+             )
+      do
+      (cond
+       ((or (boundp 'si::*gcl-major-version*) ;GCL 2.0 or greater
+            (and (boundp 'si::*gcl-version*) ;GCL 1.1
+                 (= si::*gcl-version* 1)))
+        (si::allocate-growth type 1 10 50 2))
+       (t (si::allocate-growth type 1 10 50)))))
+   (cond
+    ((or (not (probe-file *acl2-status-file*))
+         (with-open-file (str *acl2-status-file*
+                              :direction :input)
+                         (not (member (read str nil)
+                                      '(:compiled :initialized)))))
+     (error "Please compile ACL2 using ~s or~%~
             ~s, which will write the~%~
             token :compiled to the file acl2-status.txt."
-           '(compile-acl2)
-           '(quick-compile-acl2 t))))
-  (let ((*readtable* *acl2-readtable*))
-    (dolist (name *copy-of-acl2-files*)
-      (or (equal name "defpkgs")
-          (if (equal name "proof-checker-pkg")
-              (load "proof-checker-pkg.lisp")
-            (load-compiled (make-pathname :name name
-                                          :type *compiled-file-extension*)))))
-    (load "defpkgs.lisp")
-    (in-package "ACL2")
+            '(compile-acl2)
+            '(quick-compile-acl2 t))))
+   (let ((*readtable* *acl2-readtable*))
+     (dolist (name *copy-of-acl2-files*)
+       (or (equal name "defpkgs")
+           (if (equal name "proof-checker-pkg")
+               (load "proof-checker-pkg.lisp")
+             (load-compiled (make-pathname :name name
+                                           :type *compiled-file-extension*)))))
+     (load "defpkgs.lisp")
+     (in-package "ACL2")
 
 ; Do not make state special, as that can interfere with tail recursion removal.
 ; The following form is provided merely as a convenience to the user, who may
-; want to execute ACL2 forms in raw Lisp.  The eval form is there as an attempt
-; to keep Lisp systems from deciding that state should be special upon reading
-; this file.  The special case for cmulisp is to avoid getting a warning that
-; state is undefined.
+; want to execute ACL2 forms in raw Lisp.  The use of set instead of setq is to
+; avoid getting a warning in cmulisp that state is undefined.
 
-    #-cmu
-    (eval '(setq state *the-live-state*))
-    #+cmu
-    (set 'state *the-live-state*)
-    "ACL2"))
+     (set 'state *the-live-state*)
+     "ACL2")))
 
 (defun load-acl2-execution-environment ()
   (let ((*readtable* *acl2-readtable*))
@@ -1307,49 +1329,68 @@ Newline
     (in-package "ACL2")
     "ACL2"))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            DECLARATIONS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; We use XARGS in DECLARE forms.  By making this proclamation, we
 ; suppress compiler warnings.
 
-(proclaim '(declaration xargs))
+(declaim (declaration xargs))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            EXITING LISP
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun exit-lisp ()
-  #+akcl
-  (lisp::bye)
-  #+lucid
-  (lisp::exit)
+(defun exit-lisp (&optional (cmd '"(GOOD-BYE)"))
+  #-(or openmcl clisp lispworks sbcl)
+  (declare (ignore cmd))
 
-; We do not yet know why (lisp::exit) does not cause CLISP to exit while inside
-; the ACL2 loop, and similarly for (cl-user::quit) in OpenMCL.  So we print an
+; We do not yet know why the appropriate exit command does not cause an exit
+; for each of the indicated lisps while inside the ACL2 loop.  So we print ba
 ; informative message instead if we are in the loop.
 
-  #+(or clisp openmcl lispworks)
+  #+(or openmcl clisp lispworks sbcl)
   (if (and (boundp '*ld-level*)
            (not (eql 0 (symbol-value '*ld-level*))))
-      (progn (format t
-                     "To exit both ACL2 and lisp, exit the ACL2 loop with :q ~
-                      and then type (good-bye) again, or ~s <RETURN>.~%"
-                     #+clisp
-                     '(user::exit)
-                     #+openmcl
-                     '(cl-user::quit)
-                     #+lispworks ; Version 4.2.0; older versions have used bye
-                     '(lisp::quit)
-                     )
+      (progn
+        #-openmcl
+        (format t
+                "To exit both ACL2 and lisp, exit the ACL2 loop with :q~%~
+                 and then: type ~a again, or ~s, then <RETURN>.~%"
+                cmd
+                #+clisp
+                '(user::exit)
+                #+lispworks ; Version 4.2.0; older versions have used bye
+                '(lisp::quit)
+                #+sbcl
+                '(sb-ext:quit)
+                )
+        #+openmcl
+        (progn (format t
+                       "Quitting ACL2 and lisp; this may take a moment.  For~%~
+                        faster response time, exit the ACL2 loop with :q~%~
+                        and then: type ~a or ~s, then <RETURN>.~%"
+                       cmd
+                       '(ccl::quit))
+               (ccl::quit))
 
 ; Return t here so that (good-bye) does not cause an error.
 
              (return-from exit-lisp t))
     #+clisp
     (user::exit)
-    #+openmcl
-    (cl-user::quit)
     #+lispworks ; Version 4.2.0; older versions have used bye
     (lisp::quit)
+    #+sbcl
+    (sb-ext:quit)
     )
+  #+akcl
+  (lisp::bye)
+  #+lucid
+  (lisp::exit)
+  #+openmcl
+  (ccl::quit)
   #+cmu
   (common-lisp-user::quit t)
   #+allegro
@@ -1362,7 +1403,9 @@ Newline
 
   nil)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            CONSTANTS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; Allegro 6.0 (and probably later versions) warns when the following constants
 ; axioms.lisp, even with boundp checks (as in ACL2 Version_2.5).  The warning
@@ -1420,12 +1463,16 @@ Newline
 
   32)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            STATISTICS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; See the comment in *rewrite-depth-max* about rewrite stack depth:
 ; (push :acl2-rewrite-meter *features*)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;                            PROMPTS
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ; New ACL2 users sometimes do not notice that they are outside the ACL2
 ; read-eval-print loop when in a break.  See the discussion of "PROMPTS" in
@@ -1442,7 +1489,7 @@ Newline
 ; See comment about ACL2 for how the following is patched from si::break-level.
 (defun break-level-for-acl2 (at &optional env)
   (let* ((*break-message* (if (stringp at) at *break-message*))
-	 (*quit-tags* (cons (cons *break-level* *quit-tag*) *quit-tags*))
+         (*quit-tags* (cons (cons *break-level* *quit-tag*) *quit-tags*))
          (*quit-tag* (cons nil nil))
          (*break-level* (if (not at) *break-level* (cons t *break-level*)))
          (*ihs-base* (1+ *ihs-top*))
@@ -1451,11 +1498,11 @@ Newline
          (*frs-base* (or (sch-frs-base *frs-top* *ihs-base*) (1+ (frs-top))))
          (*frs-top* (frs-top))
          (*break-env* nil)
-	 (be *break-enable*)
-	 (*break-enable*
-	  (progn 
-	    (if (stringp at) nil be)))
-					;(*standard-input* *terminal-io*)
+         (be *break-enable*)
+         (*break-enable*
+          (progn 
+            (if (stringp at) nil be)))
+                                        ;(*standard-input* *terminal-io*)
          (*readtable* (or *break-readtable* *readtable*))
          (*read-suppress* nil)
          (+ +) (++ ++) (+++ +++)
@@ -1463,48 +1510,48 @@ Newline
          (* *) (** **) (*** ***)
          (/ /) (// //) (/// ///)
          )
-					; (terpri *error-output*)
+                                        ; (terpri *error-output*)
     (unless (or be (not (stringp at)))
       (simple-backtrace)
       (break-quit (length (cdr *break-level*))))
     (catch-fatal 1)
     (setq *interrupt-enable* t)
     (cond ((stringp at) (set-current)(terpri *error-output*)
-	   (setq *no-prompt* nil)
-	   )
-	  (t (set-back at env)))
+           (setq *no-prompt* nil)
+           )
+          (t (set-back at env)))
       (loop 
        (setq +++ ++ ++ + + -)
        (cond (*no-prompt* (setq *no-prompt* nil))
-	     (t ; ACL2 patch is in the following form, only
-	      (format *debug-io* "~&~a~a~a>~{~*>~}"
-		      (if (stringp at) "" "dbl:")
-		      (if (eq *package* (find-package 'user)) ""
-			(package-name *package*))
+             (t ; ACL2 patch is in the following form, only
+              (format *debug-io* "~&~a~a~a>~{~*>~}"
+                      (if (stringp at) "" "dbl:")
+                      (if (eq *package* (find-package 'user)) ""
+                        (package-name *package*))
                       *debug-prompt-suffix*
-		      *break-level*)))
+                      *break-level*)))
        (force-output *error-output*)
        (when
-	(catch 'step-continue
+        (catch 'step-continue
         (catch *quit-tag*
           (setq - (locally (declare (notinline read))
-			   (dbl-read *debug-io* nil *top-eof*)))
+                           (dbl-read *debug-io* nil *top-eof*)))
           (when (eq - *top-eof*) (bye -1))
           (let* ( break-command
-		 (values
-		  (multiple-value-list
-		  (LOCALLY (declare (notinline break-call evalhook))
-			   (if (keywordp -)(setq - (cons - nil)))
-			   (cond ((and (consp -) (keywordp (car -)))
-				  (setq break-command t)
-				  (break-call (car -) (cdr -) 'si::break-command))
-				 (t (evalhook - nil nil *break-env*)))))))
-	    (and break-command (eq (car values) :resume )(return))
+                 (values
+                  (multiple-value-list
+                  (LOCALLY (declare (notinline break-call evalhook))
+                           (if (keywordp -)(setq - (cons - nil)))
+                           (cond ((and (consp -) (keywordp (car -)))
+                                  (setq break-command t)
+                                  (break-call (car -) (cdr -) 'si::break-command))
+                                 (t (evalhook - nil nil *break-env*)))))))
+            (and break-command (eq (car values) :resume )(return))
             (setq /// // // / / values *** ** ** * * (car /))
             (fresh-line *debug-io*)
             (dolist (val /)
-		    (locally (declare (notinline prin1)) (prin1 val *debug-io*))
-		    (terpri *debug-io*)))
+                    (locally (declare (notinline prin1)) (prin1 val *debug-io*))
+                    (terpri *debug-io*)))
           nil))
         (terpri *debug-io*)
         (break-current))))))
