@@ -301,7 +301,7 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 
-(defthm crap957
+(defthm hack957
   (implies (equal (fix (eva term a)) 0)
 	   (equal (fix (eva (strip-cancelling-piece term) a)) 0)))
 
