@@ -89,7 +89,7 @@
 ; Matt K. change for v2-9: Now that terms are kept in quote-normal form, the
 ; following is illegal because the term translates to T.
 #|
-(defthm crap-minus-1
+(defthm hack-minus-1
   (not (integerp (* 1/2 -1)))
   :rule-classes :type-prescription)
 |#
