@@ -1,7 +1,7 @@
 #|
 
-   Fully Ordered Finite Sets, Version 0.9
-   Copyright (C) 2003, 2004 by Jared Davis <jared@cs.utexas.edu>
+   Fully Ordered Finite Sets, Version 0.91
+   Copyright (C) 2003-2006 by Jared Davis <jared@cs.utexas.edu>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -133,7 +133,7 @@
         ((equal (head X) (head Y)) (fast-subset (tail X) (tail Y)))
         (t (fast-subset X (tail Y)))))
 
-(defund subset (X Y)
+(defun subset (X Y)
   (declare (xargs :guard (and (setp X) (setp Y))))
   (mbe :logic (if (empty X)
 		  t
@@ -149,39 +149,50 @@
 (defund fast-measure (X Y)
   (+ (acl2-count X) (acl2-count Y)))
 
-(defund fast-union (X Y)
+(defun fast-union (x y acc)
   (declare (xargs :measure (fast-measure X Y)
-                  :guard (and (setp X) (setp Y))))
-  (cond ((empty X) Y)
-        ((empty Y) X)
-        ((equal (head X) (head Y))
-         (cons (head X) (fast-union (tail X) (tail Y))))
-        ((<< (head X) (head Y))
-         (cons (head X) (fast-union (tail X) Y)))
-        (t (cons (head Y) (fast-union X (tail Y))))))
+                  :guard (and (setp x)
+                              (setp y)
+                              (true-listp acc))))
 
-(defund fast-intersect (X Y)
-  (declare (xargs :measure (fast-measure X Y)
-                  :guard (and (setp X) (setp Y))))
-  (cond ((empty X) (sfix X))
-        ((empty Y) (sfix Y))
-        ((equal (head X) (head Y))
-         (cons (head X)
-               (fast-intersect (tail X) (tail Y))))
-        ((<< (head X) (head Y))
-         (fast-intersect (tail X) Y))
-        (t (fast-intersect X (tail Y)))))
+  (cond
+   ((empty x) (revappend acc y))
+   ((empty y) (revappend acc x))
+   ((equal (head x) (head y))
+    (fast-union (tail x) (tail y) (cons (head x) acc)))
+   ((<< (head x) (head y))
+    (fast-union (tail x) y (cons (head x) acc)))
+   (t
+    (fast-union x (tail y) (cons (head y) acc)))))
 
-(defund fast-difference (X Y)
+(defun fast-intersect (X Y acc)
   (declare (xargs :measure (fast-measure X Y)
-                  :guard (and (setp X) (setp Y))))
-  (cond ((empty X) (sfix X))
-        ((empty Y) X)
+                  :guard (and (setp X) (setp Y) (true-listp acc))))
+  (cond ((empty X) (reverse acc))
+        ((empty Y) (reverse acc))
         ((equal (head X) (head Y))
-         (fast-difference (tail X) (tail Y)))
+         (fast-intersect (tail X)
+                         (tail Y)
+                         (cons (head X) acc)))
+        
         ((<< (head X) (head Y))
-         (cons (head X) (fast-difference (tail X) Y)))
-        (t (fast-difference X (tail Y)))))
+         (fast-intersect (tail X) Y acc))
+        (t (fast-intersect X (tail Y) acc))))
+
+(defun fast-difference (X Y acc)
+  (declare (xargs :measure (fast-measure X Y)
+                  :guard (and (setp X) (setp Y) (true-listp acc))))
+
+  (cond ((empty X) (reverse acc))
+        ((empty Y) (revappend acc X))
+
+        ((equal (head X) (head Y))
+         (fast-difference (tail X) (tail Y) acc))
+
+        ((<< (head X) (head Y))
+         (fast-difference (tail X) Y (cons (head X) acc)))
+
+        (t (fast-difference X (tail Y) acc))))
 
 (defun delete (a X)
   (declare (xargs :guard (setp X)))
@@ -194,7 +205,7 @@
   (mbe :logic (if (empty X)
                   (sfix Y)
                 (insert (head X) (union (tail X) Y)))
-       :exec  (fast-union X Y)))
+       :exec  (fast-union X Y nil)))
 
 (defun intersect (X Y)
   (declare (xargs :guard (and (setp X) (setp Y))))
@@ -202,30 +213,31 @@
                     ((in (head X) Y)
                      (insert (head X) (intersect (tail X) Y)))
                     (t (intersect (tail X) Y)))
-       :exec (fast-intersect X Y)))
+       :exec (fast-intersect X Y nil)))
 
 (defun difference (X Y)
   (declare (xargs :guard (and (setp X) (setp Y))))
   (mbe :logic (cond ((empty X) (sfix X))
                     ((in (head X) Y) (difference (tail X) Y))
                     (t (insert (head X) (difference (tail X) Y))))
-       :exec (fast-difference X Y)))
+       :exec (fast-difference X Y nil)))
 
 (defun cardinality (X)
   (declare (xargs :guard (setp X)))
   (mbe :logic (if (empty X)
                   0
                 (1+ (cardinality (tail X))))
-       :exec  (len X)))
+       :exec  (length X)))
 
-(defund split-list (x)
+(defund split-list (x acc acc2)
   (declare (xargs :guard (true-listp x)))
-  (cond ((endp x) (mv nil nil))
-        ((endp (cdr x)) (mv (list (car x)) nil))
-        (t (mv-let (part1 part2)
-                   (split-list (cddr x))
-                   (mv (cons (car x) part1)
-                       (cons (cadr x) part2))))))
+  (cond ((endp x) 
+         (mv acc acc2))
+        ((endp (cdr x)) 
+         (mv (cons (car x) acc) acc2))
+        (t (split-list (cddr x)
+                       (cons (car x) acc)
+                       (cons (cadr x) acc2)))))
 
 (defun in-list (a x)
   (declare (xargs :guard (true-listp x)))
@@ -239,7 +251,7 @@
   (cond ((endp x) nil)
 	((endp (cdr x)) (insert (car x) nil))
         (t (mv-let (part1 part2)
-		   (split-list x)
+		   (split-list x nil nil)
 		   (union (mergesort-exec part1) (mergesort-exec part2))))))
 
 (defun mergesort (x)
@@ -318,7 +330,7 @@
 
 
 ; -------------------------------------------------------------------
-; Primitives Level Theorems
+; Primitive Level Theorems
 
 (defthm sets-are-true-lists
   (implies (setp X)
@@ -334,6 +346,16 @@
            (< (acl2-count (head X)) (acl2-count X)))
   :rule-classes :linear)
 
+(defthm tail-count-built-in
+  (implies (not (empty X))
+           (o< (acl2-count (tail X)) (acl2-count X)))
+  :rule-classes :built-in-clause)
+
+(defthm head-count-built-in
+  (implies (not (empty X))
+           (o< (acl2-count (head X)) (acl2-count X)))
+  :rule-classes :built-in-clause)
+
 (defthm insert-insert
   (equal (insert a (insert b X))
          (insert b (insert a X)))
@@ -414,6 +436,10 @@
   (implies (and (in a X) (not (in a (tail X))))
            (equal (head X) a)))
 
+(defthm in-head
+  (equal (in (head X) X)
+         (not (empty X))))
+
 (defthm head-unique
   (not (in (head X) (tail X))))
 
