@@ -1,5 +1,7 @@
 (in-package "ACL2")
 
+(set-enforce-redundancy t)
+
 ; Optionally, one may wish to consider:
 ; (include-book "../../../misc/rtl-untranslate")
 ; to inhibit expansion of macros in proof output.
@@ -10,11 +12,11 @@
 
 (include-book "rtlarr") ;semantics RTL array primitives
 
-(include-book "bits") ;bit vectors and logical operations
-
 (include-book "basic") ;properties of basic arithmetic functions: floor, ceiling, 
 ;                       exponential, and remainder
 
+(include-book "bits") ;bit vectors and logical operations
+
 (include-book "float") ;floating-point numbers
 
 (include-book "reps") ;floating-point formats and representations
@@ -28,6 +30,4 @@
 ; (include-book "../arithmetic/top")
 (include-book "arith") ;general arithmetic package
 
-(include-book "brat") ;support for binary notation
-
 (include-book "util") ;misc helpful stuff including a few macros
